from mocha.decorators import emit_signal

@emit_signal()
def create_user(change):
    """
    Emit a signal when a new user has been created
    Returns the USER instance
    """
    return change()

@emit_signal()
def user_update(user, action, change, data={}):
    """
    Emit when a user's account has been update. 
    Check action to see the type of update 

    :param user: The user instance
    :param action: the action taken
    :param change: a callback fn that will executed
    :param data: additional data 
    :return: a tuple (user, action, change)
    """
    return user, action, change()

@emit_signal()
def delete_user(change):
    """
    When a user has been deleted
    """
    return change()

@emit_signal()
def user_login(change):
    """
    Emit signal when logged in
    """
    return change()

@emit_signal()
def user_logout(change):
    """
    When logout
    """
    return change()






import inspect
from mocha import (register_package,
                     get_config,
                     decorators as h_deco,
                     abort)


# `contrib` prefix is set so all templates in this package
# get accessed via `contrib/`
register_package(__package__, "contrib")


# ------------------------------------------------------------------------------

# ADMIN

ADMIN_LAYOUT = "contrib/admin/layout.jade"

ADMIN_TAG = "ADMIN"

def disable_admin(*a, **kw):
    abort(404)

# @admin
def admin(f):
    """
    @admin
    A decorator that turns a class into ADMIN
    """
    import auth.decorators as a_deco

    if not inspect.isclass(f):
        raise TypeError("@ADMIN expects a Mocha class")

    if get_config("ADMIN_ENABLED", True):

        # ROLES
        min_role = get_config("ADMIN_MIN_ACL", "ADMIN")
        role_name = "accepts_%s_roles" % min_role.lower()

        if not hasattr(a_deco, role_name):
            raise ValueError("Invalid ADMIN_MIN_ACL: %s" % min_role)

        getattr(a_deco, role_name)(f)
        a_deco.login_required(f)

        f._nav_tags = [ADMIN_TAG]
        layout = get_config("ADMIN_LAYOUT") or ADMIN_LAYOUT
        return h_deco.template(layout=layout)(f)

    else:
        f._nav_visible = False
        f.before_request = disable_admin
        return f




# -*- coding: utf-8 -*-
"""
exceptions.py

Raise Mocha specific exceptions
"""

class MochaError(Exception):
    """
    This exception is not reserved, but it used for all Mocha exception.
    It helps catch Core problems.
    """
    pass


class AppError(MochaError):
    """
    Use this exception in your application level.
    """
    pass


class ModelError(MochaError):
    """
    Use this exception in your model level.
    """
    pass







# -*- coding: utf-8 -*-
""""
________________________________________________________________________________
MOCHA: https://github.com/mardix/mocha
________________________________________________________________________________
"""

from mocha import Brew

# == CLI ==
# Import your app cli. Omit if it will not be used
import application.cli


# == PROJECTS ==
# Apps is a dict with list of views that will be loaded by name
# ie: `app=main mocha :serve` will serve all the views in the `main` list
# Views are placed in application/views directory, and should be listed as string
# without the `.py`
# You can add as many projects as you want, containing as many views
# It also allows you to use multiple config env
# ie: `app=main:production mocha :serve` will use the `main` project with
# `production` config

apps = {
    "main": [
        "main"
    ]
}

# == INIT ==
# Init the application
# 'app' variable is mandatory if you plan on using the mocha cli
app = Brew(__name__, apps)




"""
views.admin is a basic home page for your admin area
"""

from mocha import (Mocha,
                   decorators as deco,
                   )

import mocha.contrib


@deco.route("/admin/")
@mocha.contrib.admin
class Admin(Mocha):

    @deco.template("contrib/admin/Admin/index.jade")
    @deco.nav_title("Admin Home", tags=mocha.contrib.ADMIN_TAG,
                    attach_to=["mocha.contrib.views.auth.Account", "self"])
    def index(self):
        return
